"""Provides the JWT bearer token validator."""

from abc import ABC, abstractmethod
from typing import Generic, TypeVar

from .objects import JWTPayload
from .types import JWTToken

JWTBearerPayloadGeneric = TypeVar("JWTBearerPayloadGeneric", bound=JWTPayload)


class JWTVerifierAbstract(ABC, Generic[JWTBearerPayloadGeneric]):
    """JWT verifier."""

    @abstractmethod
    async def verify(
        self,
        jwt_token: JWTToken,
        jwt_payload: JWTBearerPayloadGeneric,
    ) -> None:
        """Verify the JWT bearer token.

        Args:
            jwt_token (JWTToken): The JWT bearer token.
            jwt_payload (JWTBearerPayloadGeneric): The JWT bearer payload.

        Raises:
            NotVerifiedJWTError: If the JWT bearer token is not verified.
        """
        raise NotImplementedError()


class JWTNoneVerifier(JWTVerifierAbstract[JWTPayload]):
    """JWT none verifier."""

    async def verify(self, jwt_token: JWTToken, jwt_payload: JWTPayload) -> None:
        """Verify the JWT bearer token.

        Args:
            jwt_token (JWTToken): The JWT bearer token.
            jwt_payload (JWTBearerPayload): The JWT bearer payload.

        Raises:
            NotVerifiedJWTError: If the JWT bearer token is not verified.
        """
        return



"""Provides security-related functions for the API."""

from .configs import JWTBearerAuthenticationConfig
from .decoders import (
    JWTBearerTokenDecoder,
    JWTBearerTokenDecoderAbstract,
)
from .exceptions import (
    InvalidJWTError,
    InvalidJWTPayploadError,
    JWTAuthenticationError,
    MissingJWTCredentialsError,
    NotVerifiedJWTError,
)
from .objects import JWTPayload
from .services import (
    JWTAuthenticationService,
    JWTAuthenticationServiceAbstract,
)
from .stores import JWKStoreAbstract, JWKStoreMemory
from .types import OAuth2Scope
from .verifiers import JWTNoneVerifier, JWTVerifierAbstract

__all__: list[str] = [
    "InvalidJWTError",
    "InvalidJWTPayploadError",
    "JWKStoreAbstract",
    "JWKStoreMemory",
    "JWTAuthenticationError",
    "JWTAuthenticationService",
    "JWTAuthenticationServiceAbstract",
    "JWTBearerAuthenticationConfig",
    "JWTBearerTokenDecoder",
    "JWTBearerTokenDecoderAbstract",
    "JWTNoneVerifier",
    "JWTPayload",
    "JWTVerifierAbstract",
    "MissingJWTCredentialsError",
    "NotVerifiedJWTError",
    "OAuth2Scope",
]



"""Hydra service module."""

from .exceptions import HydraOperationError, HydraTokenInvalidError
from .objects import HydraTokenIntrospectObject
from .services import (
    HydraIntrospectService,
    HydraOAuth2ClientCredentialsService,
    depends_hydra_introspect_service,
    depends_hydra_oauth2_client_credentials_service,
)

__all__: list[str] = [
    "HydraIntrospectService",
    "HydraOAuth2ClientCredentialsService",
    "HydraOperationError",
    "HydraTokenIntrospectObject",
    "HydraTokenInvalidError",
    "depends_hydra_introspect_service",
    "depends_hydra_oauth2_client_credentials_service",
]



"""Provides the JWT bearer token types."""

from typing import NewType

JWTToken = NewType("JWTToken", str)
OAuth2Scope = NewType("OAuth2Scope", str)
OAuth2Audience = NewType("OAuth2Audience", str)
OAuth2Issuer = NewType("OAuth2Issuer", str)
OAuth2Subject = NewType("OAuth2Subject", str)
